@@ -2,6 +2,7 @@ import json,dateutil,uuid,inspect
 
 from fastcore.utils import *
 from fastcore.xml import *
+from fasthtml.xtend import *
 
 from types import UnionType, SimpleNamespace as ns
 from typing import Optional, get_type_hints, get_args, get_origin, Union, Mapping, TypedDict, List
@@ -13,6 +14,7 @@ from functools import wraps, partialmethod
 
 from .starlette import *
 
+__all__ = "is_typeddict is_namedtuple date snake2hyphens htmx_hdrs HtmxHeaders str2int HttpHeader form2dict RouteX RouterX FastHTML htmx_hdrs reg_re_param MiddlewareBase Beforeware get_key".split()
 
 empty = Parameter.empty
 
@@ -124,7 +126,6 @@ async def _find_p(req, arg:str, p):
     if res is empty or res is None: res = req.headers.get(snake2hyphens(arg), None)
     if res is empty or res is None: res = nested_idx(req.scope, 'session', arg) or None
     if res is empty or res is None:
-        #import pdb; pdb.set_trace()
         frm = await req.form()
         res = frm.getlist(arg)
         if res:
@@ -141,18 +142,28 @@ async def _wrap_req(req, params):
 @dataclass
 class HttpHeader: k:str;v:str
 
+def flat_xt(lst):
+    result = []
+    for item in lst:
+        if isinstance(item, (list,tuple)) and not isinstance(item, XT): result.extend(item)
+        else: result.append(item)
+    return result
+
 def _xt_resp(req, resp, hdrs, **bodykw):
+    if not isinstance(resp, (tuple,list)): resp = (resp,)
     http_hdrs,resp = partition(resp, risinstance(HttpHeader))
     http_hdrs = {o.k:str(o.v) for o in http_hdrs}
     titles,bdy = partition(resp, lambda o: getattr(o, 'tag', '')=='title')
-    if resp and 'hx-request' not in req.headers and isinstance(resp,tuple) and titles:
-        resp = Html(Head(titles[0], *hdrs), Body(bdy, **bodykw))
+    if resp and 'hx-request' not in req.headers and not any(getattr(o, 'tag', '')=='html' for o in resp):
+        if not titles: titles = [Title('FastHTML page')]
+        resp = Html(Head(titles[0], *flat_xt(hdrs)), Body(bdy, **bodykw))
     return HTMLResponse(to_xml(resp), headers=http_hdrs)
 
 def _wrap_resp(req, resp, cls, hdrs, **bodykw):
+    if isinstance(resp, FileResponse) and not os.path.exists(resp.path): raise HTTPException(404, resp.path)
     if isinstance(resp, Response): return resp
     if cls is not empty: return cls(resp)
-    if isinstance(resp, (list,tuple)): return _xt_resp(req, resp, hdrs, **bodykw)
+    if isinstance(resp, (list,tuple)) or hasattr(resp, '__xt__'): return _xt_resp(req, resp, hdrs, **bodykw)
     if isinstance(resp, str): cls = HTMLResponse
     elif isinstance(resp, Mapping): cls = JSONResponse
     else:
@@ -186,6 +197,10 @@ def _wrap_ep(f, hdrs, before, **bodykw):
         return _wrap_resp(req, resp, cls, hdrs, **bodykw)
     return _f
 
+class WS_RouteX(WebSocketRoute):
+    def __init__(self, path:str, endpoint, *, name=None, middleware=None, hdrs=None, before=None, **bodykw):
+        super().__init__(path, _wrap_ep(endpoint, hdrs, before, **bodykw), name=name, middleware=middleware)
+
 class RouteX(Route):
     def __init__(self, path:str, endpoint, *, methods=None, name=None, include_in_schema=True, middleware=None,
                 hdrs=None, before=None, **bodykw):
@@ -205,9 +220,10 @@ class RouterX(Router):
         self.routes = [o for o in self.routes if getattr(o,'methods',None)!=methods or o.path!=path]
         self.routes.append(route)
 
-htmxscr = Script(
-    src="https://unpkg.com/htmx.org@1.9.12", crossorigin="anonymous",
-    integrity="sha384-ujb1lZYygJmzgSwoxRggbCHcjc0rB2XoQrxeTUQyRjrOnlCoYta87iKBWq3EsdM2")
+htmxscr   = Script(src="https://unpkg.com/htmx.org@next/dist/htmx.min.js")
+htmxwsscr = Script(src="https://unpkg.com/htmx-ext-ws/ws.js")
+surrsrc   = Script(src="https://cdn.jsdelivr.net/gh/gnat/surreal/surreal.js")
+scopesrc  = Script(src="https://cdn.jsdelivr.net/gh/gnat/css-scope-inline/script.js")
 
 def get_key(key=None, fname='.sesskey'):
     if key: return key
@@ -221,7 +237,7 @@ def _list(o): return [] if not o else o if isinstance(o, (tuple,list)) else [o]
 
 class FastHTML(Starlette):
     def __init__(self, debug=False, routes=None, middleware=None, exception_handlers=None,
-                 on_startup=None, on_shutdown=None, lifespan=None, hdrs=None, before=None,
+                 on_startup=None, on_shutdown=None, lifespan=None, hdrs=None, before=None, default_hdrs=True,
                  secret_key=None, session_cookie='session_', max_age=365*24*3600, sess_path='/',
                  same_site='lax', sess_https_only=False, sess_domain=None, key_fname='.sesskey', **bodykw):
         middleware,before = _list(middleware),_list(before)
@@ -231,7 +247,8 @@ class FastHTML(Starlette):
                           https_only=sess_https_only, domain=sess_domain)
         middleware.append(sess)
         super().__init__(debug, routes, middleware, exception_handlers, on_startup, on_shutdown, lifespan=lifespan)
-        hdrs = list([] if hdrs is None else hdrs) + [htmxscr]
+        hdrs = list([] if hdrs is None else hdrs)
+        if default_hdrs: hdrs = [htmxscr,surrsrc,scopesrc] + hdrs
         self.router = RouterX(routes, on_startup=on_startup, on_shutdown=on_shutdown, lifespan=lifespan, hdrs=hdrs,
                               before=before, **bodykw)
 
